'use strict';

/* eslint-disable new-cap */

// This has been completely stolen from @threepointone's amazing original
// implementation which can be found here:
// https://github.com/threepointone/react-modules
// Go look at his implementation! It is far more featureful.
// All props go to Sunil! Love love love.

var template = require('babel-template');

// -----------------------------------------------------------------------------
// PRIVATES

var CODE_SPLIT_COMPONENT_NAME = 'CodeSplitComponent';

var INVALID_PATH_PROP_MSG = 'You must supply a string literal to the "path" property of the "CodeSplitComponent". It should be the relative path to the Component you would like to load and code split against. For example:\n  <CodeSplitComponent path="./Foo">\n    {\n      Foo => Foo\n        ? <Foo />\n        : <span>Loading..</span>\n    }\n  </CodeSplitComponentLoader>';

var CODE_SPLIT_TEMPLATE = template('System.import(SOURCE)');
var NO_CODE_SPLIT_TEMPLATE = template('require(SOURCE)');

var TRUE = template('true');
var FALSE = template('false');

// -----------------------------------------------------------------------------
// PLUGIN

function codeSplitComponentPlugin(_ref) {
  var t = _ref.types;

  return {
    visitor: {
      JSXElement: function JSXElement(path, state) {
        var noCodeSplitting = state.opts.noCodeSplitting;

        if (path.node.openingElement.name.name === CODE_SPLIT_COMPONENT_NAME) {
          path.node.openingElement.attributes.forEach(function (attr) {
            if (!attr.name || attr.name.name !== 'path') {
              return;
            }

            var isValid = attr.value.type && attr.value.type === 'StringLiteral';

            if (!isValid) {
              throw new Error(INVALID_PATH_PROP_MSG);
            }

            var resolvedTemplate = noCodeSplitting ? NO_CODE_SPLIT_TEMPLATE : CODE_SPLIT_TEMPLATE;

            attr.value = t.jSXExpressionContainer( // eslint-disable-line no-param-reassign
            resolvedTemplate({
              SOURCE: t.stringLiteral(attr.value.value)
            }).expression);
          });

          // Adds a transpiled={true} prop to the code split component loader.
          path.node.openingElement.attributes.push(t.jSXAttribute(t.jSXIdentifier('noCodeSplitting'), t.jSXExpressionContainer((noCodeSplitting ? TRUE() : FALSE()).expression)));
        }
      }
    }
  };
}

module.exports = codeSplitComponentPlugin;